@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -105,6 +105,7 @@ module Kramdown
       def convert_header(el, opts)
         res = ''
         res << "#{'#' * output_header_level(el.options[:level])} #{inner(el, opts)}"
+        res[-1, 1] = "\\#" if res[-1] == ?#
         res << "   {##{el.attr['id']}}" if el.attr['id'] && !el.attr['id'].strip.empty?
         res << "\n"
       end
@@ -223,16 +224,16 @@ module Kramdown
       def convert_thead(el, opts)
         rows = inner(el, opts)
         if opts[:alignment].all? {|a| a == :default}
-          "#{rows}|" + "-"*10 + "\n"
+          "#{rows}|" << "-"*10 << "\n"
         else
-          "#{rows}| " + opts[:alignment].map do |a|
+          "#{rows}| " << opts[:alignment].map do |a|
             case a
             when :left then ":-"
             when :right then "-:"
             when :center then ":-:"
             when :default then "-"
             end
-          end.join(' ') + "\n"
+          end.join(' ') << "\n"
         end
       end
 
@@ -244,11 +245,11 @@ module Kramdown
       end
 
       def convert_tfoot(el, opts)
-        "|" + "="*10 + "\n#{inner(el, opts)}"
+        "|" << "="*10 << "\n#{inner(el, opts)}"
       end
 
       def convert_tr(el, opts)
-        "| " + el.children.map {|c| convert(c, opts)}.join(" | ") + " |\n"
+        "| " << el.children.map {|c| convert(c, opts)}.join(" | ") << " |\n"
       end
 
       def convert_td(el, opts)
@@ -285,8 +286,9 @@ module Kramdown
       end
 
       def convert_img(el, opts)
+        alt_text = el.attr['alt'].gsub(ESCAPED_CHAR_RE) { $1 ? "\\#{$1}" : $2 }
         if el.attr['src'].empty?
-          "![#{el.attr['alt']}]()"
+          "![#{alt_text}]()"
         else
           title = (el.attr['title'] ? ' "' + el.attr['title'].gsub(/"/, "&quot;") + '"' : '')
           link = if el.attr['src'].count("()") > 0
@@ -294,7 +296,7 @@ module Kramdown
                  else
                    el.attr['src']
                  end
-          "![#{el.attr['alt']}](#{link}#{title})"
+          "![#{alt_text}](#{link}#{title})"
         end
       end
 
@@ -321,11 +323,13 @@ module Kramdown
       end
 
       def convert_em(el, opts)
-        "*#{inner(el, opts)}*"
+        "*#{inner(el, opts)}*" +
+          (opts[:next] && [:em, :strong].include?(opts[:next].type) && !ial_for_element(el) ? '{::}' : '')
       end
 
       def convert_strong(el, opts)
-        "**#{inner(el, opts)}**"
+        "**#{inner(el, opts)}**" +
+          (opts[:next] && [:em, :strong].include?(opts[:next].type) && !ial_for_element(el) ? '{::}' : '')
       end
 
       def convert_entity(el, opts)
@@ -404,8 +408,10 @@ module Kramdown
             " #{k}=\"#{v.to_s}\""
           end
         end.compact.join('')
-        res = "toc" + (res.strip.empty? ? '' : " #{res}") if (el.type == :ul || el.type == :ol) &&
+        res = "toc" << (res.strip.empty? ? '' : " #{res}") if (el.type == :ul || el.type == :ol) &&
           (el.options[:ial][:refs].include?('toc') rescue nil)
+        res = "footnotes" << (res.strip.empty? ? '' : " #{res}") if (el.type == :ul || el.type == :ol) &&
+          (el.options[:ial][:refs].include?('footnotes') rescue nil)
         res.strip.empty? ? nil : "{:#{res}}"
       end
 
