@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -32,8 +32,6 @@ module Kramdown
       # Initialize the LaTeX converter with the +root+ element and the conversion +options+.
       def initialize(root, options)
         super
-        #TODO: set the footnote counter at the beginning of the document
-        @options[:footnote_nr]
         @data[:packages] = Set.new
       end
 
@@ -186,7 +184,7 @@ module Kramdown
       end
 
       def convert_tr(el, opts)
-        el.children.map {|c| send("convert_#{c.type}", c, opts)}.join(' & ') + "\\\\\n"
+        el.children.map {|c| send("convert_#{c.type}", c, opts)}.join(' & ') << "\\\\\n"
       end
 
       def convert_td(el, opts)
@@ -194,7 +192,7 @@ module Kramdown
       end
 
       def convert_comment(el, opts)
-        el.value.split(/\n/).map {|l| "% #{l}"}.join("\n") + "\n"
+        el.value.split(/\n/).map {|l| "% #{l}"}.join("\n") << "\n"
       end
 
       def convert_br(el, opts)
@@ -205,7 +203,7 @@ module Kramdown
 
       def convert_a(el, opts)
         url = el.attr['href']
-        if url =~ /^#/
+        if url.start_with?('#')
           "\\hyperlink{#{escape(url[1..-1])}}{#{inner(el, opts)}}"
         else
           "\\href{#{escape(url)}}{#{inner(el, opts)}}"
@@ -213,14 +211,15 @@ module Kramdown
       end
 
       def convert_img(el, opts)
+        line = el.options[:location]
         if el.attr['src'] =~ /^(https?|ftps?):\/\//
-          warning("Cannot include non-local image")
+          warning("Cannot include non-local image#{line ? " (line #{line})" : ''}")
           ''
         elsif !el.attr['src'].empty?
           @data[:packages] << 'graphicx'
           "#{latex_link_target(el)}\\includegraphics{#{el.attr['src']}}"
         else
-          warning("Cannot include image with empty path")
+          warning("Cannot include image with empty path#{line ? " (line #{line})" : ''}")
           ''
         end
       end
