@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -68,14 +68,14 @@ module Kramdown
 
         @root.options[:abbrev_defs] = {}
         @alds = {}
-        @link_defs = {}
-        @options[:link_defs].each {|k,v| @link_defs[normalize_link_id(k)] = v}
         @footnotes = {}
+        @link_defs = {}
+        update_link_definitions(@options[:link_defs])
 
         @block_parsers = [:blank_line, :codeblock, :codeblock_fenced, :blockquote, :atx_header,
                           :horizontal_rule, :list, :definition_list, :block_html, :setext_header,
-                          :table, :footnote_definition, :link_definition, :abbrev_definition,
-                          :block_extensions, :block_math, :eob_marker, :paragraph]
+                          :block_math, :table, :footnote_definition, :link_definition, :abbrev_definition,
+                          :block_extensions, :eob_marker, :paragraph]
         @span_parsers =  [:emphasis, :codespan, :autolink, :span_html, :footnote_marker, :link, :smart_quotes, :inline_math,
                          :span_extensions, :html_entity, :typographic_syms, :line_break, :escaped_chars]
 
@@ -89,13 +89,24 @@ module Kramdown
         parse_blocks(@root, adapt_source(source))
         update_tree(@root)
         replace_abbreviations(@root)
-        @footnotes.each {|name,data| update_tree(data[:marker].last.value) if data[:marker]}
+        @footnotes.each {|name,data| update_tree(data[:content])}
       end
 
       #######
       protected
       #######
 
+      # :doc:
+      #
+      # Update the parser specific link definitions with the data from +link_defs+ (the value of the
+      # :link_defs option).
+      #
+      # The parameter +link_defs+ is a hash where the keys are possibly unnormalized link IDs and
+      # the values are two element arrays consisting of the link target and a title (can be +nil+).
+      def update_link_definitions(link_defs)
+        link_defs.each {|k,v| @link_defs[normalize_link_id(k)] = v}
+      end
+
       # Adapt the object to allow parsing like specified in the options.
       def configure_parser
         @parsers = {}
@@ -118,7 +129,8 @@ module Kramdown
       # Parse all block-level elements in +text+ into the element +el+.
       def parse_blocks(el, text = nil)
         @stack.push([@tree, @src, @block_ial])
-        @tree, @src, @block_ial = el, (text.nil? ? @src : StringScanner.new(text)), nil
+        @tree, @block_ial = el, nil
+        @src = (text.nil? ? @src : ::Kramdown::Utils::StringScanner.new(text, el.options[:location]))
 
         status = catch(:stop_block_parsing) do
           while !@src.eos?
@@ -148,7 +160,8 @@ module Kramdown
         element.children.map! do |child|
           if child.type == :raw_text
             last_blank = nil
-            reset_env(:src => StringScanner.new(child.value), :text_type => :text)
+            reset_env(:src => ::Kramdown::Utils::StringScanner.new(child.value, element.options[:location]),
+                      :text_type => :text)
             parse_spans(child)
             child.children
           elsif child.type == :eob
@@ -165,6 +178,7 @@ module Kramdown
             last_blank = nil
             update_tree(child)
             update_attr_with_ial(child.attr, child.options[:ial]) if child.options[:ial]
+            update_raw_header_text(child) if child.type == :header
             child
           end
         end.flatten!
@@ -253,6 +267,25 @@ module Kramdown
         end
       end
 
+      # Update the raw header text for automatic ID generation.
+      def update_raw_header_text(header)
+        # DEPRECATED: option auto_id_stripping will be removed in 2.0 because then this will be the
+        # default behaviour
+        return unless @options[:auto_id_stripping]
+        raw_text = ''
+
+        append_text = lambda do |child|
+          if child.type == :text
+            raw_text << child.value
+          else
+            child.children.each {|c| append_text.call(c)}
+          end
+        end
+
+        append_text.call(header)
+        header.options[:raw_text] = raw_text
+      end
+
       # Create a new block-level element, taking care of applying a preceding block IAL if it
       # exists. This method should always be used for creating a block-level element!
       def new_block_el(*args)
