@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -15,9 +15,10 @@ module Kramdown
 
       # Parse the codespan at the current scanner location.
       def parse_codespan
+        start_line_number = @src.current_line_number
         result = @src.scan(CODESPAN_DELIMITER)
         simple = (result.length == 1)
-        reset_pos = @src.pos
+        saved_pos = @src.save_pos
 
         if simple && @src.pre_match =~ /\s\Z/ && @src.match?(/\s/)
           add_text(result)
@@ -30,9 +31,9 @@ module Kramdown
             text = text[1..-1] if text[0..0] == ' '
             text = text[0..-2] if text[-1..-1] == ' '
           end
-          @tree.children << Element.new(:codespan, text)
+          @tree.children << Element.new(:codespan, text, nil, :location => start_line_number)
         else
-          @src.pos = reset_pos
+          @src.revert_pos(saved_pos)
           add_text(result)
         end
       end
