@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -15,10 +15,12 @@ module Kramdown
 
       # Parse the emphasis at the current location.
       def parse_emphasis
+        start_line_number = @src.current_line_number
+        saved_pos = @src.save_pos
+
         result = @src.scan(EMPHASIS_START)
         element = (result.length == 2 ? :strong : :em)
         type = result[0..0]
-        reset_pos = @src.pos
 
         if (type == '_' && @src.pre_match =~ /[[:alnum:]]\z/ && @src.check(/[[:alnum:]]/)) || @src.check(/\s/) ||
             @tree.type == element || @stack.any? {|el, _| el.type == element}
@@ -27,7 +29,7 @@ module Kramdown
         end
 
         sub_parse = lambda do |delim, elem|
-          el = Element.new(elem)
+          el = Element.new(elem, nil, nil, :location => start_line_number)
           stop_re = /#{Regexp.escape(delim)}/
           found = parse_spans(el, stop_re) do
             (@src.pre_match[-1, 1] !~ /\s/) &&
@@ -39,14 +41,16 @@ module Kramdown
 
         found, el, stop_re = sub_parse.call(result, element)
         if !found && element == :strong && @tree.type != :em
-          @src.pos = reset_pos - 1
+          @src.revert_pos(saved_pos)
+          @src.pos += 1
           found, el, stop_re = sub_parse.call(type, :em)
         end
         if found
           @src.scan(stop_re)
           @tree.children << el
         else
-          @src.pos = reset_pos
+          @src.revert_pos(saved_pos)
+          @src.pos += result.length
           add_text(result)
         end
       end
