@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -16,7 +16,7 @@ module Kramdown
       # Parse the string +str+ and extract all attributes and add all found attributes to the hash
       # +opts+.
       def parse_attribute_list(str, opts)
-        return if str.strip.empty?
+        return if str.strip.empty? || str.strip == ':'
         attrs = str.scan(ALD_TYPE_ANY)
         attrs.each do |key, sep, val, ref, id_and_or_class, _, _|
           if ref
@@ -54,19 +54,20 @@ module Kramdown
       # Parse the generic extension at the current point. The parameter +type+ can either be :block
       # or :span depending whether we parse a block or span extension tag.
       def parse_extension_start_tag(type)
-        orig_pos = @src.pos
+        saved_pos = @src.save_pos
+        start_line_number = @src.current_line_number
         @src.pos += @src.matched_size
 
         error_block = lambda do |msg|
           warning(msg)
-          @src.pos = orig_pos
+          @src.revert_pos(saved_pos)
           add_text(@src.getch) if type == :span
           false
         end
 
         if @src[4] || @src.matched == '{:/}'
           name = (@src[4] ? "for '#{@src[4]}' " : '')
-          return error_block.call("Invalid extension stop tag #{name}found - ignoring it")
+          return error_block.call("Invalid extension stop tag #{name} found on line #{start_line_number} - ignoring it")
         end
 
         ext = @src[1]
@@ -80,24 +81,24 @@ module Kramdown
             body = result.sub!(stop_re, '')
             body.chomp! if type == :block
           else
-            return error_block.call("No stop tag for extension '#{ext}' found - ignoring it")
+            return error_block.call("No stop tag for extension '#{ext}' found on line #{start_line_number} - ignoring it")
           end
         end
 
-        if !handle_extension(ext, opts, body, type)
-          error_block.call("Invalid extension with name '#{ext}' specified - ignoring it")
+        if !handle_extension(ext, opts, body, type, start_line_number)
+          error_block.call("Invalid extension with name '#{ext}' specified on line #{start_line_number} - ignoring it")
         else
           true
         end
       end
 
-      def handle_extension(name, opts, body, type)
+      def handle_extension(name, opts, body, type, line_no = nil)
         case name
         when 'comment'
-          @tree.children << Element.new(:comment, body, nil, :category => type) if body.kind_of?(String)
+          @tree.children << Element.new(:comment, body, nil, :category => type, :location => line_no) if body.kind_of?(String)
           true
         when 'nomarkdown'
-          @tree.children << Element.new(:raw, body, nil, :category => type, :type => opts['type'].to_s.split(/\s+/)) if body.kind_of?(String)
+          @tree.children << Element.new(:raw, body, nil, :category => type, :location => line_no, :type => opts['type'].to_s.split(/\s+/)) if body.kind_of?(String)
           true
         when 'options'
           opts.select do |k,v|
