@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #--
-# Copyright (C) 2009-2013 Thomas Leitner <t_leitner@gmx.at>
+# Copyright (C) 2009-2014 Thomas Leitner <t_leitner@gmx.at>
 #
 # This file is part of kramdown which is licensed under the MIT.
 #++
@@ -19,11 +19,12 @@ module Kramdown
 
       # Parse the foot note definition at the current location.
       def parse_footnote_definition
+        start_line_number = @src.current_line_number
         @src.pos += @src.matched_size
 
-        el = Element.new(:footnote_def)
+        el = Element.new(:footnote_def, nil, nil, :location => start_line_number)
         parse_blocks(el, @src[2].gsub(INDENT, ''))
-        warning("Duplicate footnote name '#{@src[1]}' - overwriting") if @footnotes[@src[1]]
+        warning("Duplicate footnote name '#{@src[1]}' on line #{start_line_number} - overwriting") if @footnotes[@src[1]]
         (@footnotes[@src[1]] = {})[:content] = el
         @tree.children << Element.new(:eob, :footnote_def)
         true
@@ -35,15 +36,16 @@ module Kramdown
 
       # Parse the footnote marker at the current location.
       def parse_footnote_marker
+        start_line_number = @src.current_line_number
         @src.pos += @src.matched_size
         fn_def = @footnotes[@src[1]]
         if fn_def
           fn_def[:marker] ||= []
-          fn_def[:marker].push(Element.new(:footnote, fn_def[:content], nil, :name => @src[1]))
+          fn_def[:marker].push(Element.new(:footnote, fn_def[:content], nil, :name => @src[1], :location => start_line_number))
           fn_def[:stack] = [@stack.map {|s| s.first}, @tree, fn_def[:marker]].flatten.compact
           @tree.children << fn_def[:marker].last
         else
-          warning("Footnote definition for '#{@src[1]}' not found")
+          warning("Footnote definition for '#{@src[1]}' not found on line #{start_line_number}")
           add_text(@src.matched)
         end
       end
